from setuptools import setup

setup(
  name='lavalink',
  packages=['lavalink'],
  version='2.0.2.5',
  description='A lavalink interface built for discord.py',
  author='Luke, William',
  author_email='dev@crimsonxv.pro',
  url='https://github.com/Devoxin/Lavalink.py',
  download_url='https://github.com/Devoxin/Lavalink.py/archive/2.0.2.5.tar.gz',
  keywords=['lavalink'],
)
